---@class CraftSim
local CraftSim = select(2, ...)

CraftSim.NEWS = {}

local print = CraftSim.DEBUG:SetDebugPrint(CraftSim.CONST.DEBUG_IDS.NEWS)

---@param itemMap table<string, ItemMixin>
function CraftSim.NEWS:GET_NEWS(itemMap)
    -- minimize names to make manual formatting easier :p
    local f = CraftSim.GUTIL:GetFormatter()
    local function newP(v) return f.l("\n                                   --- Version " .. v .. " ---\n") end
    local supporterListUpdate = f.p .. f.patreon("Supporter List Update ") ..
        CraftSim.MEDIA:GetAsTextIcon(CraftSim.MEDIA.IMAGES.PIXEL_HEART, 0.15)
    local news = {
        f.bb("                   Hello and thank you for using CraftSim!\n"),
        f.bb("                                 ( You are awesome! )"),
        newP("15.0.10"),
        f.P .. "Introducing " .. f.bb("Sub Recipes Optimization"),
        f.a .. "- " .. f.l("CraftSim") .. " is now able to optimize and cache",
        f.a .. "  crafting cost details for reagents",
        f.a .. "  " .. f.r("This is still experimental"),
        f.P .. f.g("Cost Details") .. " renamed to " .. f.g("Cost Optimization"),
        f.a .. "- Now shows the lowest expected crafting costs for an item",
        f.a .. "  if you or one of your alts are able to craft it (and its cached)",
        f.a .. "- Subcrafting costs are considered if you enable it in the module",
        f.a .. "- Now has a " .. f.g("Sub Recipe Options") .. " Tab to configure sub crafts",
        f.P .. f.g("Recipe Scan"),
        f.a .. "- Added a Recipe Scan Option to toggle " .. f.bb("sub crafts"),
        f.a .. "- Current cooldown charges are now shown besides",
        f.a .. "  the recipe name",
        f.a .. "- The alt profession list is now more consistently sorted",
        f.p .. f.bb("Cooldowns"),
        f.a .. "- Fixed a bug when displaying a cooldown that is fully charged",
        f.p .. f.bb("CraftResults"),
        f.a .. "- Fixed a Typo in the Item Result Log saying 'Profit: :'",
        f.a .. "  instead of 'Saved Reagents:",
        f.p .. "Russian Translation Update",
        f.a .. "- Thanks to " .. f.bb("https://github.com/SerGlushko"),
    }
    return table.concat(news, "\n")
end

---@param newsText string
function CraftSim.NEWS:GetChecksum(newsText)
    local checksum = 0
    local checkSumBitSize = 256

    -- replace each itemLink with a generic string so there are no differences between characters in the checksum
    -- for _, item in pairs(itemMap) do
    -- end
    newsText = string.gsub(newsText, "|cff%x+|Hitem:.+|h|r", "[LINK]")

    -- Iterate through each character in the string
    for i = 1, #newsText do
        checksum = (checksum + string.byte(newsText, i)) % checkSumBitSize
    end

    -- print("replacing links in newstext:")
    -- print(newsText)

    return checksum
end

---@param newsText string
---@return string | nil newChecksum newChecksum when news should be shown, otherwise nil
function CraftSim.NEWS:IsNewsUpdate(newsText)
    local newChecksum = CraftSim.NEWS:GetChecksum(newsText)
    local oldChecksum = CraftSimOptions.newsChecksum
    if newChecksum ~= oldChecksum then
        return newChecksum
    end
    return nil
end

---@param force boolean wether to skip the checksum verification
---@async
function CraftSim.NEWS:ShowNews(force)
    local itemMap = {
        chocolate = Item:CreateFromItemID(194902),
    }
    CraftSim.GUTIL:ContinueOnAllItemsLoaded(CraftSim.GUTIL:Map(itemMap, function(i) return i end), function()
        local newsText = CraftSim.NEWS:GET_NEWS(itemMap)
        local newChecksum = CraftSim.NEWS:IsNewsUpdate(newsText)
        if newChecksum == nil and (not force) then
            return
        end

        print("showing news, old / new cs: " .. tostring(CraftSimOptions.newsChecksum) .. "/" .. tostring(newChecksum))
        CraftSimOptions.newsChecksum = newChecksum

        local infoFrame = CraftSim.GGUI:GetFrame(CraftSim.MAIN.FRAMES, CraftSim.CONST.FRAMES.INFO)
        -- resize
        infoFrame:SetSize(CraftSim.CONST.infoBoxSizeX, CraftSim.CONST.infoBoxSizeY)
        infoFrame.originalX = CraftSim.CONST.infoBoxSizeX
        infoFrame.originalY = CraftSim.CONST.infoBoxSizeY
        infoFrame.showInfo(newsText)
    end)
end
